define(['app/WebkitFileApi', 'lib/localforage'], function(WebkitFileApi, localForage) {

	function LoadBinaryFile(config) {
		this.locals = {__proto__: null};
		this.transfer = {__proto__: null};
		
		this._events = {__proto__: null};
		
		this.baseUrl = '';
		
		this.id = 0;
		
		this.config(config);
		
		this.webkitFileApi = new WebkitFileApi();
		
		this.cache_root_dir = "4bb38722b63833bc6267990ceab93f8c";
		
		this.ownload = false;
		
		this.localStorage = {__proto__: null};
	}
	
	LoadBinaryFile.prototype.cacheGet = function(key, done, fail) {
		var self = this;

		if ( this.localStorage[key] ) {
			done(this.localStorage[key]);
			return;
		}
		
		var path = this.cache_root_dir + '/'+key;

		localForage.getItem(path).then(function(data) {
			if ( !data ) {
				self.webkitFileApi.read(path, done, fail);
				return;
			}
			
			done(data);
		}, function() {
			self.webkitFileApi.read(path, done, fail);
		});
	}
	LoadBinaryFile.prototype.cacheSet = function(key, arraybuffer, done, fail) {
		var self = this;
		
		var path = this.cache_root_dir + '/'+key;

		localForage.setItem(path, arraybuffer).then(function() {
			done && done();
		}, function() {
			self.webkitFileApi.write(path, arraybuffer, done, fail);
		});
	}
	
	LoadBinaryFile.prototype.addTransfer = function(transfer) {
		for(var name in transfer) {
			if ( !transfer.hasOwnProperty(name) ) { continue; }

			this.transfer[ name.toLowerCase() ] = transfer[name].toLowerCase();
		}
	}
	LoadBinaryFile.prototype.setTransfer = function(transfer) {
		this.transfer = {__proto__: null};
		this.addTransfer(transfer);
	}
	
	LoadBinaryFile.prototype.on = function(event, callback) {
		this._events[event] = this._events[event] || [];
		this._events[event].push(callback);
	}
	LoadBinaryFile.prototype.off = function(event) {
		this._events[event] = [];
	}
	LoadBinaryFile.prototype.trigger = function(event, data) {
		if ( this._events[event] ) {
			this._events[event].map(function(callback) {
				callback(data);
			});
		}
	}
	
	LoadBinaryFile.prototype.config = function(config) {
		config = config || {};
		
		if ( config.baseUrl ) {
			this.baseUrl = config.baseUrl;
		}
		
		if ( config.transfer ) {
			this.setTransfer(config.transfer);
		}
	}
	
	LoadBinaryFile.prototype.load = function(filename, doneCallback, failCallback) {
		var self = this;
		
		var id = ++this.id;



		var url;
		
		function makeError(xhr) {
			var msg = {
				id: id,
				filename: filename,
				url: url,
				xhr: xhr,
				statusText: (xhr?xhr.statusText : "Not found")
			};
			self.trigger('error', msg);
			failCallback&&failCallback(msg);			
		}
		
		var filename_key = filename.toLowerCase();
		
		if ( !this.transfer[ filename_key ] ) {
			makeError();
			return;
		}
		var cacheKey = this.transfer[ filename_key ];
		
		if ( this.ownload ) {
			this.ownload(cacheKey, function(arraybuffer) {
				var msg = {
					id: id,
					filename: filename,
					url: '[from local load]/'+cacheKey,
					xhr: null,
					data: arraybuffer,
					response: arraybuffer,
					statusText: '[from local]'
				};
				self.trigger('loadstart', msg);	

				var msg = {
					id: id,
					filename: filename,
					url: '[from local load]/'+cacheKey,
					xhr: null,
					data: arraybuffer,
					response: arraybuffer,
					statusText: '[from local]',
					loaded: '[from local]',
					total: '[from local]',
				};
				self.trigger('progress', msg);
				
				var msg = {
					id: id,
					filename: filename,
					url: '[from local load]/'+cacheKey,
					xhr: null,
					data: arraybuffer,
					response: arraybuffer,
					statusText: '[from local]'
				};
				self.trigger('load', msg);
				
				doneCallback&&doneCallback(msg);
			}, function() {
				makeError();
				return;
			});
			return;
		}
		
		
		
		this.cacheGet(cacheKey, function(arraybuffer) {
			var msg = {
				id: id,
				filename: filename,
				url: '[from cache]/'+cacheKey,
				xhr: null,
				data: arraybuffer,
				response: arraybuffer,
				statusText: '[from cache]'
			};
			self.trigger('loadstart', msg);	

			var msg = {
				id: id,
				filename: filename,
				url: '[from cache]/'+cacheKey,
				xhr: null,
				data: arraybuffer,
				response: arraybuffer,
				statusText: '[from cache]',
				loaded: '[from cache]',
				total: '[from cache]',
			};
			self.trigger('progress', msg);
			
			var msg = {
				id: id,
				filename: filename,
				url: '[from cache]/'+cacheKey,
				xhr: null,
				data: arraybuffer,
				response: arraybuffer,
				statusText: '[from cache]'
			};
			self.trigger('load', msg);
	
			doneCallback&&doneCallback(msg);
		}, function() {
			
			url = self.baseUrl + cacheKey;
			
			var xhr = new XMLHttpRequest();
			xhr.onloadstart = function() {
				var msg = {
					id: id,
					filename: filename,
					url: url,
					xhr: this,
					data: this.response,
					response: this.response,
					statusText: this.statusText
				};
				
				self.trigger('loadstart', msg);			
			}
			xhr.onload = function(error) {
				if ( this.status == 200 ) {
					var msg = {
						id: id,
						filename: filename,
						url: url,
						xhr: this,
						data: this.response,
						response: this.response,
						statusText: this.statusText
					};
					self.trigger('load', msg);
					
					doneCallback&&doneCallback(msg);
					
					
					self.cacheSet(cacheKey, this.response);
					return;
				}
			
				makeError(this);
				return;
			};
			xhr.onerror = function(error) {
				makeError(this);
				return;
			};
			xhr.onprogress = function(event) {
				var msg = {
					id: id,
					filename: filename,
					url: url,
					xhr: this,
					event: event, // event.loaded / event.total
					loaded: event.loaded,
					total: event.total,
				};
				self.trigger('progress', msg);
			}

			xhr.open("GET", url, true);
			xhr.responseType = "arraybuffer";
			xhr.send(null);			
			
		});
	}
	
	return LoadBinaryFile;

});