define([], function() {

		var requestFileSystem = false;
		var storageInfo       = false;
		if ( window ) {
			var requestFileSystem = window.requestFileSystem || window.webkitRequestFileSystem || window.mozRequestFileSystem;
		}
		if ( navigator ) {
			var storageInfo       = navigator.temporaryStorage || navigator.webkitTemporaryStorage || navigator.mozTemporaryStorage;
		}
		
		function WebkitFileApi(quotaSize) {
			var self = this;

			this.isendload = false;
			this._onload = (function(){});
			
			var oneLoad = false;
			function errorHandler(error) {
				if (oneLoad) {return;}
				
				self.fs = false;
				self.onerror&&self.onerror();
				self.isendload = true;
				self._onload();
				
				oneLoad = true;
			}
			function doneHandler(fs) {
				if (oneLoad) {return;}
				
				self.fs = fs;
				self.onload && self.onload();
				self.isendload = true;
				self._onload();
				
				oneLoad = true;
			}
			
			quotaSize = quotaSize || 32*1024*1024*1024;
			
			this.fs = false;
			
			if ( requestFileSystem && storageInfo ) {
				storageInfo.requestQuota(quotaSize, function(grantedBytes) {
					self.grantedBytes = grantedBytes;
					
					requestFileSystem(window.TEMPORARY, grantedBytes, doneHandler, errorHandler);
					
				}, errorHandler);	


				setTimeout(errorHandler, 1.5e3);				
				
			} else {
				errorHandler();
			}
			
			this.onload;
			this.onerror;
		}
		WebkitFileApi.prototype.parsePath = function(path) {
			var list = path.replace('\\', '/').split('/');

			return {
				dirs:     list.slice(0,list.length-1),
				filename: list[list.length-1],
			};
		}
		WebkitFileApi.prototype.write = function(path, arraybuffer, doneCallback, failCallback) {
			doneCallback = doneCallback || (function(){});
			failCallback = failCallback || (function(){});
				
			var self = this;
			function _() {
				if ( !self.fs ) {
					failCallback();
					return;
				}
				
				self.toFile(path, function(fileEntry) {
					fileEntry.createWriter(function(fileWriter) {
						fileWriter.onwriteend = doneCallback;
						fileWriter.onerror = failCallback;
						//fileWriter.seek(0);
						fileWriter.write( new Blob([arraybuffer]) );
					}, failCallback);
				}, failCallback);				
			}
			
			if ( self.isendload ) {
				_();
			} else {
				var _prev = self._onload;
				self._onload = function() {
					_prev();
					_();
				}
			}
		}
		WebkitFileApi.prototype.read = function(path, doneCallback, failCallback) {
			doneCallback = doneCallback || (function(){});
			failCallback = failCallback || (function(){});
				
			var self = this;
			function _() {
				if ( !self.fs ) {
					failCallback();
					return;
				}
				
				var info = self.parsePath(path);
				if ( !info.filename ) {
					failCallback();
					return;
				}
				
				self.fs.root.getFile(info.dirs.join('/') + '/' + info.filename, {exclusive: true}, function(fileEntry) {
					fileEntry.file(function(file) {
						var reader = new FileReader();

						reader.onload = function(e) {
							doneCallback( this.result );
						};
						reader.onerror = failCallback;

						reader.readAsArrayBuffer(file);
					}, failCallback);
				}, failCallback);
			}
			
			if ( self.isendload ) {
				_();
			} else {
				var _prev = self._onload;
				self._onload = function() {
					_prev();
					_();
				}
			}
		}
		WebkitFileApi.prototype.toFile = function(path, doneCallback, failCallback) {
			if ( !this.fs ) {
				errorHandler();
				return;
			}
			
			var failReturn = false;
			
			function errorHandler() {
				failReturn = true;
				failCallback&&failCallback();
			}
			
			var info = this.parsePath(path);
			if ( !info.filename ) {
				errorHandler();
				return;
			}
			
			var fs = this.fs;
			
			var dirIndex = 0;
			function doDir(entry) {
				var dirname = info.dirs[dirIndex++];
				if ( !dirname ) {
					(function() {
						entry.getFile(info.filename, {create: true, exclusive: false}, function(fileEntry) {
							doneCallback(fileEntry);
						}, errorHandler);
					})();
					return;
				}
				
				entry.getDirectory(dirname, {create: true, exclusive: false}, function(dirEntry) {
					doDir(dirEntry);
				}, errorHandler);
			}
			
			doDir(fs.root);
		}

		return WebkitFileApi;

});