define([], function() {

	var 	EQUAL_EPSILON	= 0.001;

	function DotProduct(x,y) { return ((x)[0]*(y)[0]+(x)[1]*(y)[1]+(x)[2]*(y)[2]) };

	function VectorTransform (in1, in2, out) {
		out[0] = DotProduct(in1, in2[0]) + in2[0][3];
		out[1] = DotProduct(in1, in2[1]) +	in2[1][3];
		out[2] = DotProduct(in1, in2[2]) +	in2[2][3];
	}

	function VectorCompare(v1, v2) {
		for (i=0 ; i<3 ; i++)
			if (Math.abs(v1[i]-v2[i]) > EQUAL_EPSILON)
				return false;
		return true;
	}

	function AngleQuaternion(angles, quaternion) {
		var		angle;
		var		sr, sp, sy, cr, cp, cy;

		angle = angles[2] * 0.5;
		sy = Math.sin(angle);
		cy = Math.cos(angle);
		angle = angles[1] * 0.5;
		sp = Math.sin(angle);
		cp = Math.cos(angle);
		angle = angles[0] * 0.5;
		sr = Math.sin(angle);
		cr = Math.cos(angle);

		quaternion[0] = sr*cp*cy-cr*sp*sy; // X
		quaternion[1] = cr*sp*cy+sr*cp*sy; // Y
		quaternion[2] = cr*cp*sy-sr*sp*cy; // Z
		quaternion[3] = cr*cp*cy+sr*sp*sy; // W
	}

	function QuaternionSlerp(p, q, t, qt ) {
		var i;
		var omega, cosom, sinom, sclp, sclq;

		var a = 0;
		var b = 0;
		for (i = 0; i < 4; i++) {
			a += (p[i]-q[i])*(p[i]-q[i]);
			b += (p[i]+q[i])*(p[i]+q[i]);
		}
		if (a > b) {
			for (i = 0; i < 4; i++) {
				q[i] = -q[i];
			}
		}

		cosom = p[0]*q[0] + p[1]*q[1] + p[2]*q[2] + p[3]*q[3];

		if ((1.0 + cosom) > 0.00000001) {
			if ((1.0 - cosom) > 0.00000001) {
				omega = Math.acos( cosom );
				sinom = Math.sin( omega );
				sclp = Math.sin( (1.0 - t)*omega) / sinom;
				sclq = Math.sin( t*omega ) / sinom;
			}
			else {
				sclp = 1.0 - t;
				sclq = t;
			}
			for (i = 0; i < 4; i++) {
				qt[i] = sclp * p[i] + sclq * q[i];
			}
		}
		else {
			qt[0] = -p[1];
			qt[1] = p[0];
			qt[2] = -p[3];
			qt[3] = p[2];
			sclp = Math.sin( (1.0 - t) * 0.5 * Math.PI);
			sclq = Math.sin( t * 0.5 * Math.PI);
			for (i = 0; i < 3; i++) {
				qt[i] = sclp * p[i] + sclq * qt[i];
			}
		}
	}

	function QuaternionMatrix( quaternion, matrix ) {

		matrix[0][0] = 1.0 - 2.0 * quaternion[1] * quaternion[1] - 2.0 * quaternion[2] * quaternion[2];
		matrix[1][0] = 2.0 * quaternion[0] * quaternion[1] + 2.0 * quaternion[3] * quaternion[2];
		matrix[2][0] = 2.0 * quaternion[0] * quaternion[2] - 2.0 * quaternion[3] * quaternion[1];

		matrix[0][1] = 2.0 * quaternion[0] * quaternion[1] - 2.0 * quaternion[3] * quaternion[2];
		matrix[1][1] = 1.0 - 2.0 * quaternion[0] * quaternion[0] - 2.0 * quaternion[2] * quaternion[2];
		matrix[2][1] = 2.0 * quaternion[1] * quaternion[2] + 2.0 * quaternion[3] * quaternion[0];

		matrix[0][2] = 2.0 * quaternion[0] * quaternion[2] + 2.0 * quaternion[3] * quaternion[1];
		matrix[1][2] = 2.0 * quaternion[1] * quaternion[2] - 2.0 * quaternion[3] * quaternion[0];
		matrix[2][2] = 1.0 - 2.0 * quaternion[0] * quaternion[0] - 2.0 * quaternion[1] * quaternion[1];
	}

	function R_ConcatTransforms (in1, in2, out) {
		out[0][0] = in1[0][0] * in2[0][0] + in1[0][1] * in2[1][0] +
			in1[0][2] * in2[2][0];
		out[0][1] = in1[0][0] * in2[0][1] + in1[0][1] * in2[1][1] +
			in1[0][2] * in2[2][1];
		out[0][2] = in1[0][0] * in2[0][2] + in1[0][1] * in2[1][2] +
			in1[0][2] * in2[2][2];
		out[0][3] = in1[0][0] * in2[0][3] + in1[0][1] * in2[1][3] +
			in1[0][2] * in2[2][3] + in1[0][3];
		out[1][0] = in1[1][0] * in2[0][0] + in1[1][1] * in2[1][0] +
			in1[1][2] * in2[2][0];
		out[1][1] = in1[1][0] * in2[0][1] + in1[1][1] * in2[1][1] +
			in1[1][2] * in2[2][1];
		out[1][2] = in1[1][0] * in2[0][2] + in1[1][1] * in2[1][2] +
			in1[1][2] * in2[2][2];
		out[1][3] = in1[1][0] * in2[0][3] + in1[1][1] * in2[1][3] +
			in1[1][2] * in2[2][3] + in1[1][3];
		out[2][0] = in1[2][0] * in2[0][0] + in1[2][1] * in2[1][0] +
			in1[2][2] * in2[2][0];
		out[2][1] = in1[2][0] * in2[0][1] + in1[2][1] * in2[1][1] +
			in1[2][2] * in2[2][1];
		out[2][2] = in1[2][0] * in2[0][2] + in1[2][1] * in2[1][2] +
			in1[2][2] * in2[2][2];
		out[2][3] = in1[2][0] * in2[0][3] + in1[2][1] * in2[1][3] +
			in1[2][2] * in2[2][3] + in1[2][3];
	}3

	return {
		DotProduct: DotProduct,
		VectorTransform: VectorTransform,
		VectorCompare: VectorCompare,
		AngleQuaternion: AngleQuaternion,
		QuaternionSlerp: QuaternionSlerp,
		QuaternionMatrix: QuaternionMatrix,
		R_ConcatTransforms: R_ConcatTransforms,
	};
	
});